import numpy as np
import cv2

class a_tracker:
    
    def __init__(self,frame,rect):
        hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)
        mask = cv2.inRange(hsv, np.array((0., 60., 32.)), np.array((180., 255., 255.)))
        x0, y0, x1, y1 = rect
        x1 += x0
        y1 += y0
        hsv_roi = hsv[y0:y1, x0:x1]
        mask_roi = mask[y0:y1, x0:x1]
        self.hist = cv2.calcHist( [hsv_roi], [0], mask_roi, [16], [0, 180] )
        cv2.normalize(self.hist, self.hist, 0, 255, cv2.NORM_MINMAX)
        self.hist = self.hist.reshape(-1)
        self.rect = rect

    def detect(self,frame):
        hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)
        mask = cv2.inRange(hsv, np.array((0., 60., 32.)), np.array((180., 255., 255.)))
        prob = cv2.calcBackProject([hsv], [0], self.hist, [0, 180], 1)
        prob &= mask
        term_crit = ( cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 10, 1 )
        rotatedRect, self.rect = cv2.CamShift(prob, self.rect, term_crit)
        return rotatedRect

if __name__ == "__main__":
    # selection of the followed object 
    refPt0 = refPt1 = None
    selPt0 = selPt1 = None
    def mouseHandler(event, x, y, flags, param):
        global refPt0, refPt1, selPt0, selPt1
        if event == cv2.EVENT_LBUTTONDOWN:
            refPt0 = (x, y)
        elif event == cv2.EVENT_LBUTTONUP:
            selPt0 = refPt0
            selPt1 = (x,y) 
            refPt0 = refPt1 = None
        elif refPt0 is not None:
            refPt1 = (x,y)
            
    cv2.namedWindow("image")
    cv2.setMouseCallback("image", mouseHandler)

    camera = cv2.VideoCapture(0,cv2.CAP_DSHOW)
    tracker = None
    while True:
        hasFrame, frame = camera.read() 
        if not hasFrame: 
            break
        frame = cv2.flip(frame,1) # mirror, remove this out for robot
        res = np.copy(frame)
        if selPt0 is not None and selPt1 is not None:
            tl = (min(selPt0[0],selPt1[0]),min(selPt0[1],selPt1[1]))
            br = (max(selPt0[0],selPt1[0]),max(selPt0[1],selPt1[1]))
            print(tl,br)
            target = frame[tl[1]:br[1],tl[0]:br[0]]
            if len(target) > 0:
                cv2.imshow('target',target)
                rect = (tl[0],tl[1],br[0]-tl[0]+1,br[1]-tl[1]+1)
                tracker = a_tracker(frame,rect)
            selPt0 = setPt1 = None
        elif refPt0 is not None and refPt1 is not None:
            cv2.rectangle(res,(min(refPt0[0],refPt1[0]),min(refPt0[1],refPt1[1])),(max(refPt0[0],refPt1[0]),max(refPt0[1],refPt1[1])),(0,255,0),1)
        
        if tracker is not None:
            rotatedRect = tracker.detect(frame)
            cv2.ellipse(res, rotatedRect, (0,255,0), 2)
        
        cv2.imshow("image", res)
        if cv2.waitKey(1) == 27:
            break

    camera.release()      
    cv2.destroyAllWindows()

