import numpy as np
import cv2

class a_tracker:
    
    def __init__(self,frame,rect):

        # target processing
        self.tracker = cv2.TrackerMIL_create()
        # self.tracker = cv2.TrackerKCF_create()
        self.tracker.init(frame,rect)

    def detect(self,frame):
    
        # frame processing
        success, rect = self.tracker.update(frame)

        return int(rect[0]), int(rect[1]), int(rect[2]), int(rect[3]), success


if __name__ == "__main__":
    # selection of the followed object 
    refPt0 = refPt1 = None
    selPt0 = selPt1 = None
    def mouseHandler(event, x, y, flags, param):
        global refPt0, refPt1, selPt0, selPt1
        if event == cv2.EVENT_LBUTTONDOWN:
            refPt0 = (x, y)
        elif event == cv2.EVENT_LBUTTONUP:
            selPt0 = refPt0
            selPt1 = (x,y) 
            refPt0 = refPt1 = None
        elif refPt0 is not None:
            refPt1 = (x,y)
            
    cv2.namedWindow("image")
    cv2.setMouseCallback("image", mouseHandler)

    camera = cv2.VideoCapture(0,cv2.CAP_DSHOW)
    tracker = None
    while True:
        hasFrame, frame = camera.read() 
        if not hasFrame: 
            break
        frame = cv2.flip(frame,1) # mirror, remove this out for robot
        res = np.copy(frame)
        if selPt0 is not None and selPt1 is not None:
            tl = (min(selPt0[0],selPt1[0]),min(selPt0[1],selPt1[1]))
            br = (max(selPt0[0],selPt1[0]),max(selPt0[1],selPt1[1]))
            print(tl,br)
            target = frame[tl[1]:br[1],tl[0]:br[0]]
            if len(target) > 0:
                cv2.imshow('target',target)
                rect = (tl[0],tl[1],br[0]-tl[0]+1,br[1]-tl[1]+1)
                tracker = a_tracker(frame,rect)
            selPt0 = setPt1 = None
        elif refPt0 is not None and refPt1 is not None:
            cv2.rectangle(res,(min(refPt0[0],refPt1[0]),min(refPt0[1],refPt1[1])),(max(refPt0[0],refPt1[0]),max(refPt0[1],refPt1[1])),(0,255,0),1)
        
        if tracker is not None:
            x, y, w, h, success = tracker.detect(frame)
            print(x,y,w,h,success)
            if success:
                cv2.rectangle(res,(x,y),(x+w,y+h),(0,0,255),1)
        
        cv2.imshow("image", res)
        key = cv2.waitKey(1)
        if key == 27:
            break
        elif key == ord('s'):
            cv2.imwrite('target.png',target)
            cv2.imwrite('frame.png',cv2.cvtColor(frame,cv2.COLOR_BGR2GRAY))

    camera.release()      
    cv2.destroyAllWindows()
